@@ -27,7 +27,7 @@ var Types = {
   preciseI64MathUsed: (PRECISE_I64_MATH == 2)
 };
 
-var firstTableIndex = FUNCTION_POINTER_ALIGNMENT * RESERVED_FUNCTION_POINTERS + 1;
+var firstTableIndex = RESERVED_FUNCTION_POINTERS + 1;
 
 var Functions = {
   // All functions that will be implemented in this file. Maps id to signature
@@ -126,6 +126,10 @@ var LibraryManager = {
           'library_workerfs.js',
           'library_lz4.js',
         ]);
+
+        if (NODERAWFS) {
+          libraries.push('library_noderawfs.js')
+        }
       }
     }
 
@@ -169,8 +173,9 @@ var LibraryManager = {
     for (var i = 0; i < libraries.length; i++) {
       var filename = libraries[i];
       var src = read(filename);
+      var processed = undefined;
       try {
-        var processed = processMacros(preprocess(src, filename));
+        processed = processMacros(preprocess(src, filename));
         eval(processed);
       } catch(e) {
         var details = [e, e.lineNumber ? 'line number: ' + e.lineNumber : '', (e.stack || "").toString().replace('Object.<anonymous>', filename)];
@@ -271,21 +276,165 @@ var EXPORTED_RUNTIME_METHODS_SET = set(EXPORTED_RUNTIME_METHODS.concat(EXTRA_EXP
 EXPORTED_RUNTIME_METHODS = unset(EXPORTED_RUNTIME_METHODS_SET);
 EXTRA_EXPORTED_RUNTIME_METHODS = [];
 
-function maybeExport(name) {
-  // if requested to be exported, export it
-  if (name in EXPORTED_RUNTIME_METHODS_SET) {
-    return 'Module["' + name + '"] = ' + name + ';';
+function isFSPrefixed(name) {
+  return name.length > 3 && name[0] === 'F' && name[1] === 'S' && name[2] === '_';
+}
+
+// forcing the filesystem exports a few things by default
+function isExportedByForceFilesystem(name) {
+  return name === 'FS_createFolder' ||
+         name === 'FS_createPath' ||
+         name === 'FS_createDataFile' ||
+         name === 'FS_createPreloadedFile' ||
+         name === 'FS_createLazyFile' ||
+         name === 'FS_createLink' ||
+         name === 'FS_createDevice' ||
+         name === 'FS_unlink' ||
+         name === 'getMemory' ||
+         name === 'addRunDependency' ||
+         name === 'removeRunDependency';
+}
+
+// export parts of the JS runtime that the user asked for
+function exportRuntime() {
+  // optionally export something.
+  // in ASSERTIONS mode we show a useful error if it is used without
+  // being exported. how we show the message depends on whether it's
+  // a function (almost all of them) or a number.
+  function maybeExport(name, isNumber) {
+    // if requested to be exported, export it
+    if (name in EXPORTED_RUNTIME_METHODS_SET) {
+      var exported = name;
+      if (isFSPrefixed(exported)) {
+        // this is a filesystem value, FS.x exported as FS_x
+        exported = 'FS.' + exported.substr(3);
+      }
+      return 'Module["' + name + '"] = ' + exported + ';';
+    }
+    // do not export it. but if ASSERTIONS, emit a
+    // stub with an error, so the user gets a message
+    // if it is used, that they should export it
+    if (ASSERTIONS) {
+      // check if it already exists, to support EXPORT_ALL and other cases
+      // (we could optimize this, but in ASSERTIONS mode code size doesn't
+      // matter anyhow)
+      var extra = '';
+      if (isExportedByForceFilesystem(name)) {
+        extra = '. Alternatively, forcing filesystem support (-s FORCE_FILESYSTEM=1) can export this for you';
+      }
+      if (!isNumber) {
+        return 'if (!Module["' + name + '"]) Module["' + name + '"] = function() { abort("\'' + name + '\' was not exported. add it to EXTRA_EXPORTED_RUNTIME_METHODS (see the FAQ)' + extra + '") };';
+      } else {
+        return 'if (!Module["' + name + '"]) Object.defineProperty(Module, "' + name + '", { get: function() { abort("\'' + name + '\' was not exported. add it to EXTRA_EXPORTED_RUNTIME_METHODS (see the FAQ)' + extra + '") } });';
+      }
+    }
+    return '';
   }
-  // do not export it. but if ASSERTIONS, emit a
-  // stub with an error, so the user gets a message
-  // if it is used, that they should export it
+
+  function maybeExportNumber(name) {
+    return maybeExport(name, true);
+  }
+
+  // All possible runtime elements to export
+  var runtimeElements = [
+    'intArrayFromString',
+    'intArrayToString',
+    'ccall',
+    'cwrap',
+    'setValue',
+    'getValue',
+    'allocate',
+    'getMemory',
+    'Pointer_stringify',
+    'AsciiToString',
+    'stringToAscii',
+    'UTF8ArrayToString',
+    'UTF8ToString',
+    'stringToUTF8Array',
+    'stringToUTF8',
+    'lengthBytesUTF8',
+    'UTF16ToString',
+    'stringToUTF16',
+    'lengthBytesUTF16',
+    'UTF32ToString',
+    'stringToUTF32',
+    'lengthBytesUTF32',
+    'allocateUTF8',
+    'stackTrace',
+    'addOnPreRun',
+    'addOnInit',
+    'addOnPreMain',
+    'addOnExit',
+    'addOnPostRun',
+    'writeStringToMemory',
+    'writeArrayToMemory',
+    'writeAsciiToMemory',
+    'addRunDependency',
+    'removeRunDependency',
+    'FS',
+    'FS_createFolder',
+    'FS_createPath',
+    'FS_createDataFile',
+    'FS_createPreloadedFile',
+    'FS_createLazyFile',
+    'FS_createLink',
+    'FS_createDevice',
+    'FS_unlink',
+    'GL',
+    'staticAlloc',
+    'dynamicAlloc',
+    'warnOnce',
+    'loadDynamicLibrary',
+    'loadWebAssemblyModule',
+    'getLEB',
+    'getFunctionTables',
+    'alignFunctionTables',
+    'registerFunctions',
+    'addFunction',
+    'removeFunction',
+    'getFuncWrapper',
+    'prettyPrint',
+    'makeBigInt',
+    'dynCall',
+    'getCompilerSetting',
+    'stackSave',
+    'stackRestore',
+    'stackAlloc',
+  ];
+  if (SUPPORT_BASE64_EMBEDDING) {
+    runtimeElements.push('intArrayFromBase64');
+    runtimeElements.push('tryParseAsDataURI');
+  }
+  // dynCall_* methods are not hardcoded here, as they
+  // depend on the file being compiled. check for them
+  // and add them.
+  for (var name in EXPORTED_RUNTIME_METHODS_SET) {
+    if (/^dynCall_/.test(name)) {
+      // a specific dynCall; add to the list
+      runtimeElements.push(name);
+    }
+  }
+  var runtimeNumbers = [
+    'ALLOC_NORMAL',
+    'ALLOC_STACK',
+    'ALLOC_STATIC',
+    'ALLOC_DYNAMIC',
+    'ALLOC_NONE',
+  ];
   if (ASSERTIONS) {
-    // check if it already exists, to support EXPORT_ALL and other cases
-    // (we could optimize this, but in ASSERTIONS mode code size doesn't
-    // matter anyhow)
-    return 'if (!Module["' + name + '"]) Module["' + name + '"] = function() { abort("\'' + name + '\' was not exported. add it to EXTRA_EXPORTED_RUNTIME_METHODS (see the FAQ)") };';
+    // check all exported things exist, warn about typos
+    for (var name in EXPORTED_RUNTIME_METHODS_SET) {
+      if (runtimeElements.indexOf(name) < 0 &&
+          runtimeNumbers.indexOf(name) < 0) {
+        printErr('warning: invalid item (maybe a typo?) in EXPORTED_RUNTIME_METHODS: ' + name);
+      }
+    }
   }
-  return '';
+  return runtimeElements.map(function(name) {
+    return maybeExport(name);
+  }).join('\n') + runtimeNumbers.map(function(name) {
+    return maybeExportNumber(name);
+  }).join('\n');
 }
 
 var PassManager = {
